#!/usr/bin/python

import logging
import sys
import os
import pgturant.constants

global LOGGEROBJ

def getTurantLogger(logToStdOut=True):
    """
    Return the logger handler
    """
    global LOGGEROBJ
    logName = 'PgTurant'
    logLevel = logging.INFO
    logHome =  getLogHome()
    logFormat = '%(asctime)s %(levelname)s %(message)s'

    logger = logging.getLogger(logName)
    if (len(logger.handlers) < 1):
        LOGGEROBJ = getLogger(logName, logLevel, logFormat,
            os.path.join(logHome, pgturant.constants.PGTURANT_LOG_FILENAME), False, logToStdOut)
        return LOGGEROBJ
    else:
        LOGGEROBJ = getLoggerByName(logName)
        return LOGGEROBJ

def getLogHome():
    thisFilePath = os.path.abspath( __file__ )
    thisFileDir  = os.path.dirname(thisFilePath )
    thisLogDir   = os.path.abspath(os.path.join(thisFileDir, "../../logs"))

    return thisLogDir

def getLogFilePath():
    thisFilePath = os.path.abspath( __file__ )
    thisFileDir  = os.path.dirname(thisFilePath )
    thisLogDir   = os.path.abspath(os.path.join(thisFileDir, "../../logs"))

    return os.path.join(thisLogDir, pgturant.constants.PGTURANT_LOG_FILENAME)

def getLogger(logName, logLevel, logFormat=None, logFile=None, logToStdErr =
                True, logToStdOut = True):
    """
        Return the logger with the given options
    """
    stdLogFormat = '%(pathname)s %(levelname)s %(message)s'
    hndlrs = []
    if not logFormat:
        logFormat = stdLogFormat
    logger = logging.getLogger(logName)

    if logFile:
        try:
            if not os.access(os.path.dirname(logFile), os.F_OK):
                os.makedirs(os.path.dirname(logFile))
            hndlrs.append(logging.FileHandler(logFile))
        except Exception, e:
            print "Error creating handler for logfile %s" % logFile
    if logToStdErr:
        try:
            # If we dont provide a stream to it, stderr is used
            hndlrs.append(logging.StreamHandler())
        except Exception, e:
            print "Error creating handler for std error"
    import traceback
    if logToStdOut:
        try:
            # If we dont provide a stream to it, stderr is used
            # Let's call out stdout explicitly
            hndlrs.append(logging.StreamHandler(sys.stdout))
        except Exception, e:
            print "Error creating handler for std error"

    try:
        formatter = logging.Formatter(logFormat)
    except:
        print "Invalid log format %s" % logFormat

    for hndlr in hndlrs:
        hndlr.setFormatter(formatter)
        logger.addHandler(hndlr)

    if logLevel == 'WARNING':
        logger.setLevel(logging.WARNING)
    elif logLevel == 'DEBUG':
        logger.setLevel(logging.DEBUG)
    elif logLevel == 'ERROR':
        logger.setLevel(logging.ERROR)
    elif logLevel == 'CRITICAL':
        logger.setLevel(logging.CRITICAL)
    else:
        logger.setLevel(logging.INFO)
    return logger

def getLoggerByName(logName):
    """
    Return the logger with the given name
    """
    return logging.getLogger(logName)

def display(logMsg, lineFeed=True):

    sys.stdout.write(logMsg)
    if (lineFeed):
       sys.stdout.write("\n")
    sys.stdout.flush()

def printLog(logType, logMsg, lineFeed=True):
    """ Prints program output messages both to logfile and tty """

    global LOGGEROBJ
    logFlag = True
    try:
       if LOGGEROBJ == None:
          logFlag = False
    except:
       logFlag = False

    if logType == "warn":
        display(pgturant.constants.TTY_ESC_CHARS['WARNING'] + logMsg + pgturant.constants.TTY_ESC_CHARS['ENDC'], lineFeed)
        if logFlag: LOGGEROBJ.warn(logMsg)
    elif logType == "error":
        display(pgturant.constants.TTY_ESC_CHARS['FAIL'] + logMsg + pgturant.constants.TTY_ESC_CHARS['ENDC'], lineFeed)
        if logFlag: LOGGEROBJ.error(logMsg)
    elif logType == "success":
        display(pgturant.constants.TTY_ESC_CHARS['OKGREEN'] + logMsg + pgturant.constants.TTY_ESC_CHARS['ENDC'], lineFeed)
        if logFlag: LOGGEROBJ.info(logMsg)
    elif logType == "header":
        display(pgturant.constants.TTY_ESC_CHARS['HEADER'] + logMsg + pgturant.constants.TTY_ESC_CHARS['ENDC'], lineFeed)
        if logFlag: LOGGEROBJ.info(logMsg)
    elif logType == "title":
        display(pgturant.constants.TTY_ESC_CHARS['OKBLUE'] + logMsg + pgturant.constants.TTY_ESC_CHARS['ENDC'], lineFeed)
        if logFlag: LOGGEROBJ.info(logMsg)
    elif logType == "notify" or logType == "info":
        display(logMsg, lineFeed)
        if logFlag: LOGGEROBJ.info(logMsg)
    elif logType == "debug":
        if LOGGEROBJ.level == 10:
           display(logMsg, lineFeed)
        if logFlag:
           LOGGEROBJ.info(logMsg)
    else:
        display(logMsg)
        if logFlag: LOGGEROBJ.info(logMsg, lineFeed)
