#!/usr/bin/python

import commands
import re
import os
import platform
import pgturant.constants as constants
import pgturant.log as logger
import subprocess
import select
import sys

class Info:

   def __init__(self):

       self.initialize()
       return

   def __getitem__(self, key):

       if self._config.has_key(key):
          return self._config[key]
       return None

   def _setConfig(self, key, value):
       self._config[key] = value

   def _getConfig(self, key):
       return self._config[key]

   def initialize(self):

       self._config = {}
       self.system  = System()
       self._setConfig('arch', platform.processor())
       self._osRelease()

   def _osRelease(self):

       (osDist, osRelease, discard1) = platform.dist()
       osDist = osDist.lower()
       try:
          baseRelease = re.match(r"(\d+).*", osRelease).group(1)
       except:
          baseRelease = ''
          pass
       self._setConfig('dist', osDist)
       self._setConfig('osrelease', osRelease)
       self._setConfig('baserelease', baseRelease)

class System:

   def __init__(self):

       self.initialize()
       return

   def initialize(self):
       thisFilePath = os.path.abspath( __file__ )
       thisFileDir  = os.path.dirname(thisFilePath )
       self.rpmBasePath = os.path.abspath(os.path.join(thisFileDir, "../../rpms"))

   def isProcessRunning(self, procName):

       cmd = "ps xau 2> /dev/null |grep '%s' | grep -v 'grep' | wc -l"%procName
       try:
          ret = self.executeCommand(cmd)
          if (int(ret[1]) > 0):
             return True
       except Exception, e:
         return False
       return False

   def fixOwnerships(self, dirPath, user, group):

       try:
          logger.printLog("debug", "Fixing the ownership for [%s] as [%s:%s] recursively"%(dirPath, user, group))
          cmd = "chown %s:%s '%s' -R"%(user, group, dirPath)
          ret = self.executeCommand(cmd)
          if (ret[0] != 0):
             logger.printLog("debug", "Fixing ownership for [%s] to [%s,%s] failed!!"%(dirPath, user, group))
             logger.printLog("debug", "Reason: [%s]"%ret[1])
             return False
       except Exception, e:
          logger.printLog("debug", "Fixing ownership for [%s] to [%s,%s] failed!!"%(dirPath, user, group))
          logger.printLog("debug", "Reason: [%s]"%ret[1])
          return False

       logger.printLog("debug", "Fixing the ownership done successfully!!")

       return True

   def fixPermissions(self, dirPath, permMode):

       try:
          logger.printLog("debug", "Fixing the permission for [%s] as [%s] recursively"%(dirPath, permMode))
          cmd = "chmod %s '%s' -R"%(permMode, dirPath)
          ret = self.executeCommand(cmd)
          if (ret[0] != 0):
             logger.printLog("debug", "Fixing permission for [%s] to [%s] failed!!"%(dirPath, permMode))
             logger.printLog("debug", "Reason: [%s]"%ret[1])
             return False
       except Exception, e:
          logger.printLog("debug", "Fixing permission for [%s] to [%s] failed!!"%(dirPath, permMode))
          logger.printLog("debug", "Reason: [%s]"%ret[1])
          return False

       logger.printLog("debug", "Fixing the permission done successfully!!")

       return True

   def createDir(self, dirPath):

       try:
          logger.printLog("debug", "Creating directory [%s] ..."%dirPath)
          if (not os.path.isdir(dirPath)):
             cmd = "mkdir -p %s"%dirPath
             ret = self.executeCommand(cmd)
             if (ret[0] != 0):
                logger.printLog("debug", "Create directory [%s] failed!!"%dirPath)
                logger.printLog("debug", "Reason: [%s]"%ret[1])
                return False
          else:
             logger.printLog("debug", "Directory [%s] already exists. Skipping!"%dirPath)
       except Exception, e:
          logger.printLog("debug", "Create directory [%s] failed!!"%dirPath)
          logger.printLog("debug", "Reason: [%s]"%str(e))
          return False

       return True

   def installRpms(self, profilePath, packageList):

       finalRpmPath = os.path.join(self.rpmBasePath, profilePath)
       logger.printLog("debug", "Check and Installing these rpms %s"%packageList)
       logger.printLog("debug", "=================================")
       try:
          cmd = "yum install %s/*.rpm -y"%finalRpmPath
          ret = self.executeCommand(cmd)
          logger.printLog("debug", "Command returned [%s]"%ret[0])
          logger.printLog("debug", "Output:\n[%s]"%ret[1])
       except:
          pass
       logger.printLog("debug", "=================================")
       return ret[0]

   def baseName(self, pathString):

       pathString = re.sub(r'\s+$','', pathString)
       pathString = re.sub(r'\/$','', pathString)

       return os.path.basename(pathString)

   def dirName(self, pathString):

       pathString = re.sub(r'\s+$','', pathString)
       pathString = re.sub(r'\/$','', pathString)

       return os.path.dirname(pathString)

   def getFileContent(self, readFileName):

       cmd = "cat %s"%readFileName
       try:
         ret = self.executeCommand(cmd)
         output = ret[1]
         return output
       except Exception, e:
         return ''

   def getCommandOutput(self, cmd):

       try:
         ret = self.executeCommand(cmd)
         output = ret[1]
         return output
       except Exception, e:
         return ''

   def executeCommand(self, cmd):

       logger.printLog("debug", "system: Executing command [%s]"%cmd)
       ret = commands.getstatusoutput(cmd)

       logger.printLog("debug", "system: Command exit code [%s]"%ret[0])
       logger.printLog("debug", "system: Command output [%s]"%ret[1])
       return ret

   def executeCommandPopen(self, cmd):

       # create a child process, we want the stderr as a sepearte stream
       logger.printLog("debug", "system: Executing command [%s]"%cmd)
       child = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stdin=None, stderr=subprocess.PIPE)
       childOut = ''
       childErr = ''
       while True:
         output_line = self._readProcData(child.stdout)
         child.stdout.flush()
         if (len(output_line) != 0): logger.printLog("debug",output_line, False)
         childOut += output_line
         if child.poll() != None:
            break

       while True:
         output_line = child.stderr.readline()
         if output_line == '' and child.poll() != None:
             break
         if (len(output_line) != 0): logger.printLog("debug",output_line, False)
         childErr += output_line

       err = child.returncode
       return err,childOut,childErr

   def _readProcData(self, fObj):

       (r, w, x) = select.select([fObj], [], [], 0.2)
       value = ""
       while r:
           char = fObj.read(1)
           if (len(char) == 0):
              break
           value += char;
           (r, w, x) = select.select([fObj], [], [], 0.2)

       return value;
